library(shiny)

tdp <- paste("<td><font size=\"+1\"><strong>",sep="")
plus1 <- paste("<font size=\"+1\">")

shinyServer(function(input, output) {


   data <- reactive({
        x <- as.numeric(c(input$n11,input$n12,input$n21,input$n22))
        if(is.na(mean(x))) return(list(nodata=TRUE))
        O <- matrix(x,2,2,byrow=T)
        N <- sum(O)
        R <- apply(O,1,sum)
        S <- apply(O,2,sum)
        E <- round((cbind(R) %*% rbind(S))/N,3) 
        a <- round((O-E)^2/E,3)
        chi <- sum(a)
        list(O=O,R=R,S=S,N=sum(O),E=E,a=a,chi=chi,nodata=FALSE)
   })

   output$table0 <- renderText({
      if(data()$nodata) return("<h3>Enter the Data<h3>")
      O <- data()$O
      
      a <- rep("",5)  
      a[1] <- "<table border>"
      a[2] <- paste("<tr><th colspan=2>",plus1,"Table of Counts</th></tr>")
      a[3] <- paste("<Tr align=\"right\">",tdp,O[1,1],"</Td>",tdp,O[1,2],"</Td></Tr>")
      a[4] <- paste("<Tr align=\"right\">",tdp,O[2,1],"</Td>",tdp,O[2,2],"</td></Tr>")
      a[5] <- "</table>"
      a
   })  

   output$table1 <- renderText({  
      if(data()$nodata) return("")
      O <- data()$O
      N <- data()$N
      R <-  data()$R  
      S <- data()$S
      
      a <- rep("",7)
      a[1] <- paste("<table border=\"1\" ")
      a[2] <- paste("<tr><th colspan=3>",plus1,"Table of Counts</th></tr>")
      a[3] <- paste("<tr><th colspan=3>",plus1,"with row and column totals</th></tr>")
      a[4] <- paste("<Tr align=\"right\">",tdp,O[1,1],"</Td>",tdp,O[1,2],"</Td>",tdp,R[1],"</td></Tr>")
      a[5] <- paste("<Tr align=\"right\">",tdp,O[2,1],"</Td>",tdp,O[2,2],"</Td>",tdp,R[2],"</td></Tr>")
      a[6] <- paste("<Tr align=\"right\">",tdp,S[1],"</Td>",tdp,S[2],"</Td>",tdp,N,"</td></Tr>")
      a[7] <- "</table>"
      a
   })  

   output$table2 <- renderText({
      if(data()$nodata) return("")
      O <- data()$O
      E <- data()$E
      
      a <- rep("",5)
      a[1] <- paste("<table border=\"1\" ")
      a[2] <- paste("<tr><th colspan=3>",plus1,"Observed, Expected</th></tr>")        
      a[3] <- paste("<Tr align=\"right\">",tdp,O[1,1],", ",E[1,1],"</Td>",tdp,O[1,2],", ",E[1,2],"</Td></Tr>")
      a[4] <- paste("<Tr align=\"right\">",tdp,O[2,1],", ",E[2,1],"</Td>",tdp,O[2,2],", ",E[2,2],"</Td></Tr>")      
      a[5] <- "</table>"
      a
   }) 
   

   output$table3 <- renderText({
      if(data()$nodata) return("")
      N <- data()$N
      R <- data()$R
      S <- data()$S
      E <- data()$E
      
      a <- rep("",7)
      a[1] <- "<table>"
      a[2] <- paste("<tr><th>",plus1,"Calculation of Expected Counts:</th></tr>"        )
      a[3] <- paste("<Tr>",tdp,"E<sub>1,1</sub> = R<sub>1</sub>*S<sub>1</sub>/N =",R[1],"*",S[1],"/",N," = ",E[1,1],"</td></tr>")
      a[4] <- paste("<Tr>",tdp,"E<sub>1,2</sub> = R<sub>1</sub>*S<sub>2</sub>/N =",R[1],"*",S[2],"/",N," = ",E[1,2],"</td></tr>")
      a[5] <- paste("<Tr>",tdp,"E<sub>2,1</sub> = R<sub>2</sub>*S<sub>1</sub>/N =",R[2],"*",S[1],"/",N," = ",E[2,1],"</td></tr>")
      a[6] <- paste("<Tr>",tdp,"E<sub>2,2</sub> = R<sub>2</sub>*S<sub>2</sub>/N =",R[2],"*",S[2],"/",N," = ",E[2,2],"</td></tr>")
      a[7] <- "</table>"
      a
   })                 


   output$table4 <- renderText({
      if(data()$nodata) return("")
      N <- data()$N
      O <- data()$O
      E <- data()$E
      chivec <- data()$a
      chi <- data()$chi
      
      a <- rep("",13)
      a[1] <- "<table>"
      a[2] <- paste("<tr><th>",plus1,"Calculation of ",HTML("&chi;<sup>2</sup>")," and p-value:</th></tr>"        )
      a[3] <- paste("<Tr>",tdp,HTML("&chi;<sup>2</sup>")," =</td></tr>")
      a[4] <- paste("<Tr>",tdp,"(",O[1,1],"-",E[1,1],")<sup>2</sup>/(",E[1,1],") +,</td></tr>")
      a[5] <- paste("<Tr>",tdp,"(",O[1,2],"-",E[1,2],")<sup>2</sup>/(",E[1,2],") +,</td></tr>")
      a[6] <- paste("<Tr>",tdp,"(",O[2,1],"-",E[2,1],")<sup>2</sup>/(",E[2,1],") +,</td></tr>")
      a[7] <- paste("<Tr>",tdp,"(",O[2,2],"-",E[2,2],")<sup>2</sup>/(",E[2,2],") =,</td></tr>") 
      a[8] <- "<Tr><td>&nbsp;</td></tr>"                 
      a[9] <- paste("<Tr>",tdp,chivec[1]," + ",chivec[2]," + ",chivec[3]," + ",chivec[4]," = ",chi,"</td></tr>")
      a[10] <- "<Tr><td>&nbsp;</td></tr>"                 
      a[11] <- paste("<Tr>",tdp,"p-value = ",round(1-pchisq(chi,1),3),"</td></tr>")
      a[12] <- "</table>"
      a
   })                 

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the Chisquare test of Independence for a 2x2 Table </h5>"
        
  })

})

