library(shiny)

shinyServer(function(input, output,session) {
            
   data <- reactive({
        x <- seq(0,10,length=100)
        if(input$which=="Linear Model") {
            for(i in input$step) {
                y <- 10 + x + rnorm(100) + input$step*x^2/25
                return(data.frame(x=x,y=y))
            }
        }        
        if(input$which=="Normal Residuals") {
            for(i in input$step) {
                y <- 10 + x + 2*rt(100,11-input$step)
                return(data.frame(x=x,y=y))
            }
        }        
        if(input$which=="Equal Variance") {
            for(i in input$step) {
                y <- 10 + rnorm(100,x,1+x*input$step)  + rnorm(100) 
                return(data.frame(x=x,y=y))
            }
        }        
            
   })     
    
    
   output$plot1 <- renderPlot({
        x <- data()$x
        y <- data()$y
        fit <- lm(y~x)
        par(mfrow=c(1,2))
        plot(x,y,pch=20)
        abline(fit,lwd=3)
        if(input$which=="Normal Residuals") {
            qqnorm(residuals(fit,pch="",main=""))
            qqline(residuals(fit))
        }    
        else {
          plot(fitted(fit),residuals(fit),xlab="Fitted Values",ylab="Residuals",pch=20)
          abline(h=0,lwd=3)
        }   
   })
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the assumptions of least squares regression
        <br>for each assumption it runs a movie, starting with the case where the assumption is fullfilled
        <br>and making things worse and worse</h5>"
  })

})

