library(shiny)

shinyServer(function(input, output,session) {

    datasetInput <- reactive({ 
          x <- switch(input$dataset,
           "Babe Ruth's Homeruns" = c(54, 59, 35, 41, 46, 25, 47, 60, 54, 46, 49, 46, 41, 34, 22),
           "Gender Ratios in US" = c(47.1, 48.0, 48.2, 48.2, 48.3, 48.3, 48.3, 48.3, 48.4, 48.4,
                                    48.5, 48.6, 48.6, 48.6, 48.6, 48.6, 48.7, 48.7, 48.8, 48.8,
                                    48.9, 49.0, 49.0, 49.0, 49.0, 49.0, 49.0, 49.1, 49.1, 49.2,
                                    49.2, 49.2, 49.3, 49.4, 49.4, 49.5, 49.6, 49.6, 49.6, 49.8,
                                    49.8, 49.8, 49.9, 49.9, 50.1, 50.1, 50.2, 50.3, 50.4, 50.9,51.7),
           "Newcomb's Speed of Light" = source("newcomb.R")$value,
           "Weight of Euro Coins" = source("euros.R")$value,
           "Length of Babies" = source("cocain.R")$value
          ) 
          
          if(input$dataset == "Weight of Euro Coins" & input$europart!="All") {
                k <- as.numeric(input$europart)
                print(k)
                x <- x[((k-1)*250+1):(250*k)]
          }
          
          if(input$dataset == "Random") {
                j <- 0
                if(input$randompart=="Anything") {
                      j <- sample(1:4,1)
                }
                if(j==1 | input$randompart=="N(10,3)") x <- round(rnorm(input$n,10,3),2)
                if(j==2 | input$randompart=="t(2)") x <- round(rt(input$n,2),2)
                if(j==3 | input$randompart=="Exp(1)") x <- round(rexp(input$n,1),2)    
                if(j==4 | input$randompart=="Binomial(100,0.5)") x <- rbinom(input$n,100,0.5)                             
          }
          if(input$dataset == "Length of Babies") {
                if(input$cocainpart=="All") x <- unlist(x)
                if(input$cocainpart=="Drug Free") x <- x$'Drug Free'
                if(input$cocainpart=="First Trimester") x <- x$'First Trimester'
                if(input$cocainpart=="Throughout") x <- x$Throughout
          }          
          x
    })        
       
   perc <- function(x,p) {   
       loc <- ceiling(length(x)*p/100)
       sort(x)[loc]       
   }     
 
   data <- reactive({
        x <- datasetInput()
        m <- min(x)
        M <- max(x)
        Q1 <- perc(x,25)
        Q3 <- perc(x,75)
        Med <- median(x)
        IQR <- Q3-Q1
        LF <- Q1-1.5*IQR
        UF <- Q3+1.5*IQR
        data=list(x=x,m=m,Q1=Q1,Med=Med,Q3=Q3,M=M,IQR=IQR,LF=LF,UF=UF)
   })     
   doPlot <- list(
      function(x) {},
      function(x) {},
      function(x) {
          plot(c(x$m, x$M), c(0, 1), axes = F, xlab = "", ylab = "", type = "n")  
          axis(1)
      },    
      function(x) rect(x$Q1,0.3,x$Q3,0.8,lwd=2),
      function(x) segments(x$Med,0.3,x$Med,0.8,lwd=2),
      function(x) segments(x$Q1,0.5,ifelse(x$LF<x$m,x$m,x$LF),0.5,lwd=2),
      function(x) {
        if(x$LF>x$m) {
          x1 <- x$x[x$x<x$LF]
          for(i in x1) points(i,0.5,pch="*",cex=2)
        }
      },
      function(x) segments(x$Q3,0.5,ifelse(x$UF>x$M,x$M,x$UF),0.5,lwd=2),
      function(x) {
        if(x$UF<x$M) {
          x1 <- x$x[x$x>x$UF]
          for(i in x1) points(i,0.5,pch="*",cex=2)          
        }
      },
      function(x) {}  
    )  
    

   output$table1 <- renderText({
      x <- data()
      x1 <- prettyNum(x)
      a=rep("",12)
      
      a[1] <- "<h4>Five Number Summary</h4>"
      a[2] <- "<table border width=400>"
      a[3] <- "<TR><TH>Minimum</TH><TH>Q<sub>1</sub></TH><TH>Median</TH><TH>Q<sub>3</sub></TH><TH>Maximum</TH></TR>"
      a[4] <- paste("<TR><TH>",x1$m,"</TH>
                       <TH>",x1$Q1,"</TH>
                       <TH>",x1$Med,"</TH>
                       <TH>",x1$Q3,"</TH>
                       <TH>",x1$M,"</TH></TR>",collapse="")
      a[5] <- "</table>"                 
      a[6] <- paste("<h4>Interquartile Range: IQR = Q<sub>3</sub>-Q<sub>1</sub> =",
                      x1$Q3,"-",x1$Q1,"=",x1$IQR,"</h4>",collapse="")     
      a[7] <- paste("<h4>Lower Fence : LF = Q<sub>1</sub>-1.5*IQR = ",
                      x1$Q1,"-1.5*",x1$IQR,"=",x1$LF,"</h4>",collapse="")     
      a[8] <- paste("<h4>Is LF < Minimum ?</h4>",collapse="")           
      a[9] <- paste("<h4>",ifelse(x$LF<x$m,"Yes - draw left line to Minimum","No - draw left line to LF"),"</h4>",collapse="")                       
      a[10] <- paste("<h4>Upper Fence : UF = Q<sub>3</sub>+1.5*IQR = ",
                      x1$Q3,"+1.5*",x1$IQR,"=",x1$UF,"</h4>",collapse="")              
      a[11] <- paste("<h4>Is UF > Maximum?</h4>",collapse="")           
      a[12] <- paste("<h4>",ifelse(x$UF>x$M,"Yes - draw right line to Maximum","No - draw right line to UF"),"</h4>",collapse="")                             
      
      if(as.numeric(input$step)==0) table1 <- ""
      else table1 <- a
      table1
   })  
   
   output$text1 <- renderText({
          x <- data()
          txt <- c("Click to start ...",
            "Calculate Numbers ...",
            "Draw Scale ...",
            "Add box ...",
            "Add Median line ...",
            "Add lower fence ...",
            ifelse(x$LF>x$m,"Add outliers on left ...","No outliers on left"),
            "Add upper fence ...",
            ifelse(x$UF<x$M,"Add outliers on right ...","No outliers on right"),
            "DONE!")[as.numeric(input$step)+1]
          red <- paste("<font color=\"red\">",collapse="")  
          if(as.numeric(input$step)<=8) 
              text1 <-paste("<h3>",red,txt,"</h3>",collapse=" ")  
          else
              text1 <-paste("<h3>",txt,"</h3>",collapse=" ")    
          text1    
   
   })        
   
   output$table2 <- renderText({
          x <- prettyNum(sort(data()$x))
          line <- ""
          if(input$dataset=="Babe Ruth's Homeruns") line <- "Data: Number of homeruns hit by Babe Ruth while he 
                was with the New York Yankees (1920 - 1934)"
          if(input$dataset=="Newcomb's Speed of Light") line <- "Data: Simon Newcomb made a series of measurements of the 
                          speed of light between July and September 1882. He measured the time in seconds that a 
                          light signal took to pass from his laboratory on the Potomac River to a mirror at the 
                          base of the Washington Monument and back, a total distance of 7400m. 
                          His first measurement was 0.000024828 seconds, or 24,828 nanoseconds 
                          (10<sup>9</sup> nanoseconds = 1 second). The data are the deviations (differences) 
                          from 24,800 nanoseconds"
          if(input$dataset=="Weight of Euro Coins") line <- "Data: Weights of 2000 1-euro coins, 250 each in
                eight rolls. The data were collected by Herman Callaert at Hasselt University in Belgium. The 
                euro coins were borrowed at a local bank. Two assistants, Sofie Bogaerts and Saskia Litiere 
                weighted the coins one by one, in laboratory conditions on a weighing scale of the type Sartorius BP 310s."                
          if(input$dataset=="Length of Babies") line <- "Data: Chasnoff and others obtained several measures and responses for 
                newborn babies whose mothers were classified by degree of cocain use. The study was conducted in the 
                Perinatal Center for Chemical Dependence at Northwestern University Medical School. The measurement given here 
                is the length of the newborn"
          if(input$dataset=="Gender Ratios in US") line <- "Data: Sex ratios by state, without Puerto Rico. Data from the American Community Survey 2004"                
          if(input$dataset=="Random") {
              if(input$randompart=="Anything") line <- "Data: randomly chosen any of the distributions"          
              if(input$randompart=="N(10,3)") line <- "Data: randomly chosen from a Normal distribution with mean 10 and standard deviation 3"           
              if(input$randompart=="t(2)") line <- "Data: randomly chosen from a t distribution with 2 degrees of freedom"                         
              if(input$randompart=="Exp(1)") line <- "Data: randomly chosen from an Exponential distribution with rate 1"
              if(input$randompart=="Binomial(100,0.5)") line <- "Data: Number of heads in 100 flips of a fair coin"                      
          }    
          a=rep("",15)    
          n <- length(x)
          if(n<=100) a[1] <- "<h4>Ordered Dataset</h4>"
          else  a[1] <- "<h4>Part of Ordered Dataset</h4>"
          a[2] <- paste("<table CLASS=\"boldtable\" cellpadding=\"4\">",collapse="")
          a[15] <- "</table>"
          skipline <- FALSE
          if(n>100) {
              x <- c(x[1:50],x[(n-50+1):n])
              skipline <- TRUE
              n <- 100
          }       
          plus1 <- paste("<font size=\"+1\">",collapse="")
          k <- 0
          for(i in 3:13) {
              a[i] <- "<TR>"
              for(j in 1:10) {
                  k <- k+1
                  if(i==8 & skipline) a[8] <- "<TD colspan=10>...</TD>"
                  else a[i] <- paste(a[i],"<TD>",plus1,x[k],"</TD>",collapse="")
                  if(k==n) break
              }
              a[i] <- paste(a[i],"</TR>",collapse="")
              if(k==n) break    
         }    
         a <- c(a[1:i],a[15])
         c(line,a)
   })
    
   output$mybox <- renderPlot({
        for(i in 0:input$step) doPlot[[i+1]](data())
   })
           
})

