library(shiny)

shinyServer(function(input, output,session) {

    M <- 1e4
    plus1 <- paste("<font size=\"+1\">")
    data <- reactive({   
           
          if(input$dataset == "Bernoulli") 
              x <- (rbinom(M,input$n,input$p)-input$n*input$p)/sqrt(input$n*input$p*(1-input$p))
          if(input$dataset == "Exponential")     
              x <- (apply(matrix(rexp(M*input$n,1),nrow=M),1,mean)-1)*sqrt(input$n)
          if(input$dataset == "t")     
              x <- apply(matrix(rt(M*input$n,4),nrow=M),1,mean)*sqrt(input$n)/sqrt(2)
          x    

    })             

   output$table1 <- renderText({
        x <- data()
        A <- matrix(0,13,3)
        A[,1] <- c(-3,-2.5,-2,-1.5,-1,-0.5,0,0.5,1,1.5,2,2.4,3)
        for(i in 1:13) {
            z <- A[i,1]
            if(input$dataset == "Bernoulli") z <- z+0.5/sqrt(input$n)
            A[i,2] <- round(length(x[x<z])/length(x),3)
        }    
        A[,3] <- round(pnorm(A[,1]),3)
        tbl0 <- "<h3>Probabilities of the form P(X&le;x)</h3>"   
        tbl1 <- paste("<table border>")
        tra <- paste("<Tr align=\"right\">")
        tbl2 <- paste(tra,"<TH>",plus1,"x &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</TH><TH>",plus1,"Sample Mean&nbsp;</TH><TH>",plus1,"Standard Normal</TH><TH>",plus1,"Difference</TH></TR>")
        tbl <- rep("",13)
        for(i in 1:13) 
              tbl[i] <-paste(tra,"<Td><strong>",plus1,A[i,1],"</Td><TD><strong>"
              ,plus1,A[i,2],"</Td><TD><strong>",plus1,A[i,3],"</TD><TD><strong>",plus1,round(A[i,3]-A[i,2],3),"</td></TR>")
            
        c(tbl0,tbl1,tbl2,tbl,"</table>")            
   })    
   
    
   output$plot1 <- renderPlot({
        par(mfrow=c(2,2))
        par(oma=c(0,0,2,0))
        hist(data(),100,xlab="Sample Means",main="",ylab="")
        qqnorm(data())
        boxplot(data(),horizontal=T)
        mtext(paste("Mean of n =",input$n," observations"),outer=T,side=3,cex=2)        
   })
           
   output$text1 <- renderText({ 
         if(input$dataset == "Bernoulli")
            return("<h4>Bernoulli is a discrete random variable, so the graphs look a little strange!</h4>")        
   })


   output$explainText <- renderText({ 
         "<h5>Explanation: This demo illustrates the Central Limit Theorem </h5>"        
   })

})

