library(shiny)

shinyServer(function(input, output) {
       
  data <- reactive({
      m <- 100
      x <- matrix(rnorm(input$n*m,input$mu,input$sigma),nrow=m)
      xbar <- apply(x,1,mean)
      s <- apply(x,1,sd)
      z <- qnorm(1-(1-as.numeric(input$alpha)/100)/2)
      low <- xbar-z*s/sqrt(input$n)
      high <- xbar+z*s/sqrt(input$n)
      check <- ifelse(low>input$mu | high<input$mu,"Bad","Good")
      A <- data.frame('Sample Mean'=round(xbar,1),'Standard Deviation'=round(s,1),Low=round(low,1),High=round(high,1),Check=check)
      A
    })

    html_data <- reactive({
        x <- data()

        x1 <- sprintf("%.1f",sum(x[[5]]=="Good")/length(x[[5]])*100)
        a1 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;Percentage of good intervals:",x1,"%</h4>",collapse="")
        x2 <- sprintf("%.1f",mean(x[,4]-x[,3]))
        a2 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;Mean length of intervals:",x2,"</h4>",collapse="")
        x3 <- sprintf("%.1f",input$mu)
        a3 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;Population mean:",x3,"</h4>",collapse="")
        
        n <- dim(x)[1]
        m <- dim(x)[2]
        top <- "<TABLE BORDER width=400>"
        nameline <- "<TR><TH>Sample Mean</TH>
                         <TH>Std. Dev</TH>
                         <TH>Left End</TH>
                         <TH>Right End</TH>
                         <TH>Good Interval?</TH></TR>"

        middle <- rep("",n)
        tdr <- paste("<TD align=\"right\">")
        red <- paste("<font color=red>")
        plus1 <- paste("<font size=\"+1\">")
        for (j in 1:n) {
             if(x[j,5]=="Good") 
                  middle[j] <- paste("<TR>",tdr,sprintf("%.1f", x[j, 1]),"</TD>", 
                                tdr,sprintf("%.1f", x[j, 2]),"</TD>",
                                tdr,sprintf("%.1f", x[j, 3]),"</TD>",
                                tdr,sprintf("%.1f", x[j, 4]),"</TD>",
                                tdr,x[j, 5],"</TD>",
                                "</TR>",collapse="")
             else  
                  middle[j] <- paste("<TR>",tdr,sprintf("%.1f",x[j, 1]),"</TD>", 
                                tdr,sprintf("%.1f", x[j, 2]),"</TD>",
                                tdr,red,sprintf("%.1f",x[j, 3]),"</TD>",
                                tdr,red,sprintf("%.1f",x[j, 4]),"</TD>",
                                tdr,red,plus1,x[j, 5],"</TD>",
                                "</TR>",collapse="")
        }
        bottom <- "</Table>"
        c(a1,a2,a3,top,nameline,middle,bottom)
   })    
      
   output$table1 <- renderText({
        html_data()
   })      
        

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the concept of
             a confidence interval<br> </h5>"
        
  })
           
})

