library(shiny)

shinyServer(function(input, output) {

    red <- paste("<font color=\"red\">")             
    blue <- paste("<font color=\"blue\">")           
    plus1 <- paste("<font size=\"+1\">")           

    digits <- reactive({
        if(input$which=="Type I") digits<-3
        if(input$which=="Type II") digits<-4
        if(input$which=="Type III") digits<-5
        if(input$which=="Type IV") digits<-2
        digits
    })    

    data <- reactive({ 
         if(input$which=="Type I") {
            x <- rnorm(input$n,10,3)
            y <- runif(1,-10,10)+rnorm(1,0,1)*x+rnorm(input$n,0,1)
         }       
         if(input$which=="Type II") {
            x <- rnorm(input$n,30,5)
            y <- runif(1,0,60)+rnorm(1,0,5)*x+rnorm(input$n,0,5)   
         }   
         if(input$which=="Type III") {
            x <- rnorm(input$n,0.5,0.1)
            y <- runif(1,0,2)+rnorm(1,0,0.5)*x+rnorm(input$n,0,0.5)            
         }
         if(input$which=="Type IV") {
            x <- rnorm(input$n,200,50)
            y <- runif(1,100,300)+rnorm(1,0,5)*x+rnorm(input$n,0,500)
         }
        
         cbind(round(x,digits()-2),round(y,digits()-2))
    })              
 
  
   output$datatable <- renderText({
      x <- data()[,1]
      y <- data()[,2]
      lines <- "<table border>"
      k <- 0
      i <- 0
      repeat {
          i <- i+2
          lines[i] <- paste("<Tr><Th>",plus1,"x</th>")
          lines[i+1] <- paste("<Tr><Th>",plus1,"y</th>")
          for(j in 1:10) {
              k <- k+1
              if(k>length(x)) break
              lines[i] <- paste(lines[i],"<Td><strong>",HTML("&nbsp;"),x[k]," </Td>",collapse="")
              lines[i+1] <- paste(lines[i+1],"<Td><strong>",HTML("&nbsp;"),y[k]," </Td>",collapse="")
          }    
          lines[i] <- paste(lines[i],"</Tr>")
          lines[i+1] <- paste(lines[i+1],"</Tr>")
          if(k>=length(x)) break
      }    
      lines[i+2] <- "</table>"      
      lines
   }) 
   
   output$text1 <- renderText({
        x <- data()[,1]
        y <- data()[,2]
        mu_x <- mean(x)
        tbl <- matrix(0,length(x),6)
        tbl[,1] <- x
        tbl[,2] <- round(x-mu_x,digits())
        tbl[,3] <- round((x-mu_x)/sd(x),digits())
        mu_y <- mean(y)
        tbl[,4] <- y
        tbl[,5] <- round(y-mu_y,digits())
        tbl[,6] <- round((y-mu_y)/sd(y),digits())
            
        line <- "<table border>"
        line[2] <- paste("<Tr><Th><strong>",plus1,"&nbsp;&nbsp;x</th>
                <th><strong>",plus1,"&nbsp;&nbsp;",HTML("x-X&#773;"),"</th>
                <th><strong>",plus1,"&nbsp;&nbsp;z<sub>x</sub></th>
                <Th><strong>",plus1,"&nbsp;&nbsp;y</th>
                <th><strong>",plus1,"&nbsp;&nbsp;",HTML("y-Y&#773;"),"</th>
                <th><strong>",plus1,"&nbsp;&nbsp;z<sub>y</sub></th>
                </tr>")
        for(i in 1:length(x)) 
            line[i+2] <- paste("<Tr><Th><strong>",plus1,"&nbsp;&nbsp;",tbl[i,1],"</th>
                <th><strong>",plus1,"&nbsp;&nbsp;",tbl[i,2],"</th>
                <th><strong>",plus1,"&nbsp;&nbsp;",tbl[i,3],"</th>
                <Th><strong>",plus1,"&nbsp;&nbsp;",tbl[i,4],"</th>
                <th><strong>",plus1,"&nbsp;&nbsp;",tbl[i,5],"</th>
                <th><strong>",plus1,"&nbsp;&nbsp;",tbl[i,6],"</th>
                </tr>") 
            
        c(line,"</table>")    
   })        
   
   output$text2 <- renderText({
        x <- data()[,1]
        mu_x <- round(mean(x),digits())
        sd_x <- round(sd(x),digits())
        y <- data()[,2]
        mu_y <- round(mean(y),digits())
        sd_y <- round(sd(y),digits())
        line <- paste("<h3>",HTML("X&#773;"),"=",mu_x,
              HTML("&nbsp; Y&#773;"),"=",mu_y,"</h3>")
        line[2] <- paste("<h3>s<sub>x</sub>=",sd_x,
              "&nbsp;s<sub>y</sub>=",sd_y,"</h3>")
        line            
  })      
  
   output$text3 <- renderText({
        x <- data()[,1]
        y <- data()[,2]
        mu_x <- round(mean(x),digits())
        sd_x <- round(sd(x),digits())
        mu_y <- round(mean(y),digits())
        sd_y <- round(sd(y),digits())
        r <- round(sum((x-mu_x)/sd_x*(y-mu_y)/sd_y)/(length(x)-1),3)
        b1 <- round(r*sd_y/sd_x,digits())
        sgn <- ifelse(b1<0,"-","+")
        b0 <- round(mu_y-b1*mu_x,digits())
        line <- paste("<h3>r = (&Sum;z<sub>x</sub>z<sub>y</sub>)/(n-1)
              = ",r,"</h3>")
        line[2] <- paste("<h3>b<sub>1</sub> = 
                  r*s<sub>y</sub>/s<sub>x</sub> = ",
                  r,"*",sd_y,"/",sd_x," = ",b1,"</h3>")
        line[3] <- paste("<h3>b<sub>0</sub> = ", 
                  HTML("Y&#773;")," - b<sub>1</sub>",HTML("X&#773;"),
                  " = ",mu_y," - ",b1,"*",mu_x," = ",b0,"</h3>")
        line[4] <- paste("<h3>Least Squares Regression Line: y =",
                  round(b0,digits()-1),sgn,abs(round(b1,digits()-1)),"x</h3>")
        line
  })        
  
  output$plot <- renderPlot ({
      x <- data()[,1]
      y <- data()[,2]
      plot(x,y,xlab="x",ylab="y",pch=20)
      if(input$fit=="Yes")  abline(lm(y~x,lwd=3))
  })         

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo shows how to calculate a 
        correlation coefficient and the least squares regression line</h5>"
        
  })

})

