library(shiny)

shinyServer(function(input, output) {
       
  data <- reactive({
    a <- input$rho/sqrt(1-input$rho^2)
    x <- rnorm(input$n)
    y <- a*x+rnorm(input$n)
    if(input$out=="Yes") {
        x[1:input$m] <- runif(input$m,-3,-2)
        y[1:input$m] <- runif(input$m,2,3)
    }    
    cbind(x,y)
  })
        
  output$myPlot <- renderPlot({
    x <- data()[,1]
    y <- data()[,2]
    plot(x,y,pch=20,xlab="x",ylab="y")
    if(input$out=="Yes") 
        points(x[1:input$m],y[1:input$m],pch=20,cex=2,col="red")
  })
    
  output$text1 <- renderText({  
        pc <- paste(sprintf("%0.3f",input$rho))
        l1 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp;Parameter Correlation Coefficient: ",HTML("&rho;")," = ",pc,"<h4>",collapse="")
        sc <- paste(sprintf("%0.3f",cor(data())[1,2]))
        l2 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp;Sample Correlation Coefficient r = ",sc,"</h4>",collapse="")
        c(l1,l2)  
       })
  output$text2 <- renderText({
        pval <- cor.test(data()[,1],data()[,2])$p.value
        sgn <- ifelse(pval>0.05," &gt; "," &lt; ")
        rej <- ifelse(pval>0.05," fail to reject "," reject ")
        dec <- ifelse(pval>0.05," X and Y are uncorreleated "
                    ," X and Y are correlated ")
        txt <- "<h5>1) Parameter: Pearson's Correlation Coefficient &rho;</h5>
                <h5>2) Method: Standard Test</h5>"
        txt[2] <- "<h5>3) Assumption: Data comes from Normal Distribution</h5>"
        if(input$out=="No") 
            txt[3] <- "<h5>Assumption is ok</h5>"
        else txt[3] <- "<h5>Assumption is NOT ok</h5>"    
        txt[4] <- "<h5>4) &alpha;  = 0.05</h5>"
        txt[5] <- "<h5>5) H<sub>0</sub>: &rho;  = 0</h5>"
        txt[6] <- "<h5>5) H<sub>0</sub>: &rho;  &ne; 0</h5>" 
        txt[7] <- paste("<h5>7) p =",round(pval,4),"</h5>",sep="")        
        txt[8] <- paste("<h5>8) ",round(pval,4),sgn," &alpha;, so
                  we",rej," the null hypothesis, ",dec,"</h5>",sep="")
        txt            
  })      
    
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates Pearsons's Correlation Coefficient </h5>"
        
  })
})

