library(shiny)

plus1 <- paste("<font size=\"+1\">")

shinyServer(function(input, output,session) {

   data <- reactive({
      n <- 1e3
      if(input$ex=="First six in rolls of a die")  { 
         x <- sample(1:6,size=7*n,replace=T)
         z <- cumsum(ifelse(x==6,1,0))
         x <- x[z<=n]
      }   
      if(input$ex=="First Heads in flip of a coin")  {
         x <- sample(c("H","T"),size=3*n,replace=T)
         z <- cumsum(ifelse(x=="H",1,0))
         x <- x[z<=n]
      }   
      if(input$ex=="General Geometric")  {
         x <- sample(c("S","F"),size=3*n,replace=T,prob=c(input$p,1-input$p))
         z <- cumsum(ifelse(x=="S",1,0))
         x <- x[z<=n]
      }         
      x
   })
    
   pars <- reactive({
     if(input$ex=="First six in rolls of a die") {
        whatstop <- 6  
        p <- 1/6  
        rnames <- c("Rolls of a Die: ","Number of Rolls until six: ")
     }
     if(input$ex=="First Heads in flip of a coin") {
        whatstop <- "H"
        p <- 1/2 
        rnames <- c("Flips of a coin: ","Number of flips until Heads: ")
     }
     if(input$ex=="General Geometric") {
        whatstop <- "S"
        p <- input$p
        rnames <- c("Experiment: ","Number of trials until Success: ")
     }  
     list(whatstop,p,rnames)
   })  
   
   tbl1 <- reactive({
     rnames <- pars()[[3]]                            
     x <- data()
     first <- 1
     last <- 0
     txt1 <- ""
     k <- 0
     repeat {
         last <- last+1
         if(last==length(x)) break
         k <- k+1
         txt1[k] <- paste("<h3>",rnames[1],paste(x[first:last],collapse=" "),"</h3><h3>&nbsp;</h3>")
         if(x[last]==pars()[[1]]) {
                xnum <- last-first+1
                txt1[k] <- paste("<h3>",rnames[1],paste(x[first:last],collapse=" "),
                    "</h3><h3>",rnames[2],"X =",xnum,"</h3>")
                k <- k+1
                txt1[k] <- txt1[k-1]
                first <- last+1    
        }
      }
      txt1
   })     

   tbl2 <- reactive({
     x <- data()   
     maxcol <- qgeom(0.95,pars()[[2]])+1
     counter <- rep(0,maxcol)
     top <- paste("<table border width=300><tr>",
     paste("<th>",plus1,1:maxcol,"</th>",collapse=""),"</tr>")
     bottom <- "</table>"
        
     txt2 <- paste("<h3>&nbsp;</h3>",top,"<tr>",paste("<th>",plus1,counter,"</th>",
             collapse=""),"</tr>",bottom)
     first <- 1
     last <- 0
     k <- 0
     repeat {
         last <- last+1
         if(last==length(x)) break
         if(x[last]==pars()[[1]]) {
                xnum <- last-first+1
                if(xnum<=maxcol) counter[xnum] <- counter[xnum]+1
                k <- k+1
                txt2[k] <- paste("<h3>X =",xnum,"</h3>",top,"<tr>",paste("<th>",plus1,counter,"</th>",
                        collapse=""),"</tr>",bottom)
                first <- last+1
        }
      }
      txt2
   })     
   
   tbl3 <- reactive ({ 
     maxcol <- qgeom(0.99,pars()[[2]])+1
     x <- rgeom(1e3,pars()[[2]])+1
     p <-  pars()[[2]]
     probtheory <- round(dgeom(0:(maxcol-1),p),3)
     params <- round(c(1/p,sqrt(1-p)/p,qgeom(0.9,p)+1),3)
     stats  <- round(c(mean(x),sd(x),quantile(x,0.9)),3)
     counter <- rep(0,maxcol)
     for(i in 1:maxcol) counter[i] <- sum(x==i)
          probsample <- round(counter/sum(counter),3)
          line <- "<table border width=300>"
          line[2] <- paste("<tr><th>",plus1,"x</th>",paste("<th>",plus1,1:maxcol,
                              "</th>",collapse=""),"</tr>")        
          line[3] <- paste("<tr><th>",plus1,"Counts</th>",paste("<th>",plus1,counter,
                              "</th>",collapse=""),"</tr>")
          line[4] <- paste("<tr><th>",plus1,"Rel. Frequency</th>",paste("<th>",plus1,probsample,
                           "</th>",collapse=""),"</tr>")
          line[5] <- paste("<tr><th>",plus1,"Prob. Theory</th>",paste("<th>",plus1,probtheory,
                           "</th>",collapse=""),"</tr>")
          line[6] <- "</table>"  
          line[7] <- paste("<h4>Sample Mean=",stats[1],
                       "&nbsp; Population Mean=",params[1],"</h4>")               
          line[8] <- paste("<h4>Sample Standard Deviation=",stats[2],
                           "&nbsp; Population Standard Deviation=",params[2],"</h4>")               
          line[9] <- paste("<h4>Sample 90<sup>th</sup> Percentile=",stats[3],
                           "&nbsp; Population 90<sup>th</sup> Percentile=",params[3],"</h4>")                                                 
          line
   })
   
   output$head <- renderText({
      if(input$ex=="First six in rolls of a die")  
          txt <- "<h2>First six in rolls of a fair Die</h2>"
      if(input$ex=="First Heads in flip of a coin")
          txt <- "<h2>First Heads in flips of a fair Coin</h2>"
      if(input$ex=="General Geometric")    
          txt <- paste("<h2>General Geometric with p =",input$p,"</h2>")          
      txt    
          
   })          
   output$text1 <- renderText({
          if(input$speed!="slow") return("")
          for(i in 1:input$step1) text <- tbl1()[i]
          text
   })
     
   output$text2 <- renderText({
          if(input$speed!="fast") return("")
          for(i in 1:input$step2) text <- tbl2()[i]
          text   
   })      
   
   output$text3 <- renderText({
          if(input$speed!="immediate") return("")
          tbl3()
   })         
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the experiment Roll a fair die until the first six
        <br>or more generally the Geometric distribution</h5>"
        
  })

})

