library(shiny)


shinyServer(function(input, output) {
       
  data <- reactive({
      x <- runif(input$n)
      y <- input$intercept+input$slope*x+rnorm(input$n,0,input$sd)
      data.frame(x,y)
  })
        
  output$myPlot <- renderPlot({
      plot(data(),pch=20)
      abline(lm(y~x, data=data()),lwd=2,col="red")
  })
    
  output$text1 <- renderText({ 
      fit <- lm(y~x, data=data())
      cf <- round(coef(fit),2)
      r2 <- round(summary(fit)$r.squared*100,1)
      sgn=ifelse(sign(cf[2])==1,"+","-")
      line <- paste("<h4>Least Squares Regression Equation:",HTML("y&#770;")," = ",cf[1],sgn,abs(cf[2]),"x</h4>",collapse="")
      
      cf <- round(c(input$intercept,input$slope),2)
      sgn=ifelse(sign(cf[2])==1,"+","-")
      line[2] <- paste("<h4>True Line: y = ",cf[1],sgn,abs(cf[2]),"x</h4>",collapse="")
      if(sum(cf)==0) 
          line[2] <- "<h4>True Line: y = 0</h4>"
      else {
            if(cf[1]==0)  line[2] <- paste("<h4>True Line: y = ",cf[2],"x</h4>",collapse="")
            if(cf[2]==0)  line[2] <- paste("<h4>True Line: y = ",cf[1],"</h4>",collapse="")
      }            
      line[3] <- paste("<h4>R<sup>2</sup> = ",r2,"%</h4>",collapse="")
      line
     })
     
     output$explainText <- renderText({ 
         "<h5>Move the sliders around to see different cases of true lines and  least squares regression equations
         <br>Some interesting things to check:
         <br>How does changing the sample size effect things?
         <br>How does changing the residual standard deviation change things?</h5>" 
     })

})

