library(shiny)

shinyServer(function(input, output,session) {          


   param <- reactive({
        
        if(input$ex=="1") {
            coef<-c(0,1,0,0)
            sigma <- 0.1
        }
        if(input$ex=="2") {
            coef<-c(10,1,10,0)
            sigma <- 0.1
        }    
        if(input$ex=="3") {
            coef<-c(10,5,-4,0)
            sigma <- 1
        } 
        if(input$ex=="4") {
            coef<-c(10,3,-13,10)
            sigma <- 0.09
        }         
        if(input$ex==0) {
            coef <- as.numeric(c(input$beta0,input$beta1,input$beta2,input$beta3))
            sigma <- input$sigma
        }
        if(input$ex=="5") {
            coef <- rep(0,4)
            sigma <- 1
        }    
        c(coef,sigma)
   })
         
   f <- reactive({
        x <- seq(input$xrange[1],input$xrange[2],length=200)
        coef <- param()
        y <- coef[1]+coef[2]*x+coef[3]*x^2+coef[4]*x^3
        if(input$ex=="5") 
           y <- 10*(x-0.02)*(x-0.4)*(x-0.6)*(x-0.95)+0.1*x
                        
        cbind(x=x,y=y)
   })     
   
   data <- reactive({
        x <- seq(input$xrange[1],input$xrange[2],length=input$n)
        coef <- param()
        y <- coef[1]+coef[2]*x+coef[3]*x^2+coef[4]*x^3+rnorm(input$n,0,coef[5])
        if(input$ex=="5") 
           y <- 10*(x-0.02)*(x-0.4)*(x-0.6)*(x-0.95)+0.1*x+rnorm(1000,0,0.05)           
        cbind(x=x,y=y)
   }) 
   
   calcs <- reactive({
         x <- data()[,1]  
         y <- data()[,2]
         pvals <- rep(0,4)
         fit <- lm(y~x)
         pvals[1] <- round(summary(fit)[4]$coefficients[2,4],4)          
         fit <- lm(y~poly(x,2))
         pvals[2] <- round(summary(fit)[4]$coefficients[3,4],4) 
         fit <- lm(y~poly(x,3))
         pvals[3] <- round(summary(fit)[4]$coefficients[4,4],4) 
         fit <- lm(y~poly(x,4))
         pvals[4] <- round(summary(fit)[4]$coefficients[5,4],4) 
         
         pvals 
   })
        
   output$text1 <- renderText({
         pvals <- calcs() 
         line <- "<h4>Check if constant model is enough</h5>"
         line[2] <- "<h4>fit y vs x</h4>" 
         line[3] <- paste("<h4>p-value of linear term :",pvals[1],ifelse(pvals[1]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[1]>0.05) {
            line[4] <- "<h4>Linear term is not significant, constant model good</h5>"
            return(line)
         }
         line[4] <-"<hr>"
         line[5] <- paste("<h4>fit y vs x and x<sup>2</sup></h4>") 

         line[6] <- paste("<h4>p-value of quadratic term :",pvals[2],ifelse(pvals[2]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[2]>0.05) {
            line[7] <- "<h4>Quadratic term is not significant, linear model good</h5>"
            return(line)
         }
         line[7] <-"<hr>"
         line[8] <- paste("<h4>fit y vs x, x<sup>2</sup> and x<sup>3</sup></h4>") 
         line[9] <- paste("<h4>p-value of cubic term :",pvals[3],ifelse(pvals[3]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[3]>0.05) {
            line[10] <- "<h4>Cubic term is not significant, quadratic model good</h4>"
            return(line)
         }
         line[10] <-"<hr>"
         line[11] <- paste("<h4>fit y vs x, x<sup>2</sup> and x<sup>3</sup></h4>") 
         line[12] <- paste("<h4>p-value of power 4 term :",pvals[4],ifelse(pvals[4]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[4]>0.05) {
            line[13] <- "<h4>Power 4 term is not significant, cubic model good</h4>"
            return(line)
         }
         
         line[13] <- "<h4>Model should be at least of degree 4!</h4>"
         line
            
   })           
    
   output$plot1 <- renderPlot({
        par(mfrow=c(2,2))
        x <- f()
        plot(x[,1],x[,2],type="l",lwd=3,xlab="x",ylab="y",main="True Curve")
        x <- data()[,1]
        y <- data()[,2]
        plot(x,y,pch=20,lwd=3,xlab="x",ylab="y",main="Data")
        pvals <- calcs()
        if(max(pvals)<0.05) return(NULL)
        deg <- c(1:4)[pvals>0.05][1]-1

        if(deg==0) x <- data.frame(rep(1,length(x)))
        if(deg==1) {
            x <- as.numeric(x) 
            fit <- lm(y~x)
        }    
        if(deg>1) fit <- lm(y~poly(x,deg))

         plot(fitted(fit),residuals(fit),xlab="Fits",ylab="Residuals")
         abline(h=0,lwd=2)
   })   
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the concept of polynomial regression<br> and overfitting</h5>"
        
  })

})

