library(shiny)

shinyServer(function(input, output) {
       
  powercurve <- reactive({
     al <- as.numeric(input$alpha)/2 
     p <- seq(0.01,0.99,length=1000)
     y <- pbinom(qbinom(al,input$n,input$p0),input$n,p)+1-
          pbinom(qbinom(1-al/2,input$n,input$p0),input$n,p)
     i <- which.min(abs(p-input$p))     
     list(x=p,y=y,i=i)
  })
  
  simdata <- reactive({
      x <- rbinom(1e5,input$n,input$p)
      al <- as.numeric(input$alpha)/2
      ll <- qbinom(c(al,1-al),input$n,input$p0)
      I <- ifelse(x<ll[1],1,0) + ifelse(x >ll[2],1,0)
      I <- ifelse(I>0,TRUE,FALSE)
      rejected <- sum(I)
      list(x=x,I=I,rejected=rejected,limits=ll)
  })    
         
  output$plot <- renderPlot({
      par(mfrow=c(2,1))
      pc <- data.frame(p=powercurve()[[1]],y=powercurve()[[2]])
      plot(pc,type="l",lwd=2,xlab="p",ylab="Power")
      i <- powercurve()[[3]]           
      points(pc[i,],pch="X",cex=1.5)
      x1 <- simdata()[[1]]
      I <- simdata()[[2]]
      ll <- simdata()[[4]]
      hist(x1,100,xlim=range(c(x1,ll)),main="",xlab="X",ylab="")
      hist(x1[I], 100, col="blue", add=T)
      hist(x1[!I], 100, col="red", add=T)
      box()
      abline(v=ll,lwd=2)

  })
    
  output$text1 <- renderText({ 
        i <- powercurve()[[3]]
        line <- rep("",5)
        line[1] <- paste("<h4>True Population Success Probability p :",prettyNum(input$p),"</h4>",collapse="")
        line[2] <- paste("<h4>Testing H<sub>0</sub>: p=",prettyNum(input$p0)," vs H<sub>a</sub>: p&ne;",prettyNum(input$p0),"</h4>",collapse="")
        red <- paste("<font color=\"red\">")
        blue <- paste("<font color=\"blue\">")
        if(input$p==input$p0) line[3] <- paste("<h4>",blue,"Null Hypothesis is true!</font></h4>")
        else line[3] <- paste("<h4>",red,"Null Hypothesis is false!</font></h4>")
        ll <- simdata()$limits
        line[4] <- paste("<h4>Reject H<sub>0</sub> if X<",sprintf("%0.0f",ll[1])," or X>",sprintf("%0.0f",ll[2]),"</h4>",collapse="")      
        line[5] <- paste("<h4>Power of Test: ",sprintf("%0.1f",powercurve()[[2]][i]*100),"%</h4>",collapse="")
        line[6] <- paste("<h4>Number of rejected null hypotheses :",simdata()[[3]]," of 100000 or ",sprintf("%0.1f",simdata()[[3]]/1000),"%</h4>",collapse="")
        line
  })

  output$ExplainText <- renderText({ 
        line <- rep("",12)
        line[1] <- paste("<h5>Explanation: This demo illustrates the concept of the power of a hypothesis test</h5>",collapse="")
        line[2] <- paste("<h5>We carry out a sequence of independent Bernoulli trials with success probability p</h5>",collapse="")
        line[3] <- paste("<h5>We observe X successes</h5>",collapse="")
        line[4] <- paste("<h5>We want to do a test for H<sub>0</sub>: p=p<sub>0</sub> vs H<sub>a</sub>: p&ne;p<sub>0</sub></h5>",collapse="")
        line[5] <- paste("<h5>Specific Example: say we choose p<sub>0</sub>=0.5. We have n=100 events and we test at the",HTML("&alpha;"),"= 5% level</h5>",collapse="")
        line[6] <- paste("<h5>If in reality p=0.6, the null hypothesis is wrong and should be rejected.</h5>",collapse="")
        line[7] <- paste("<h5>Move sliders to: Number of Events=100, p=0.6, Hypothesized p=0.5</h5>",collapse="")  
        line[8] <- paste("<h5>shows that we should reject H<sub>0</sub> if X&lt;40 or X&gt;60</h5>",collapse="")  
        line[9] <- paste("<h5>it also show that if p=0.6 this happens 46.3% of the time</h5>",collapse="")  
        line[10] <- paste("<h5>The histogram has 10000 Monte Carlo experiments with n=100 and p=0.6</h5>",collapse="")  
        line[11] <- paste("<h5>and 46% of them are &gt; 60!</h5>",collapse="")
        line[12] <- paste("<h5></h5>",collapse="")
        line
  })


})

