library(shiny)

shinyServer(function(input, output,session) {

   data <- reactive({
        x <- matrix(rnorm(input$n*1e4,input$mu),nrow=1e4)
        xbar <- apply(x,1,mean)    
        s <- apply(x,1,sd)    
        z <- sqrt(input$n)*(xbar-10)/s
        2*(1-pt(abs(z),input$n-1))
   })     
   
 
   output$text1 <- renderText({
      x <- data()
      al <- as.numeric(input$alpha)
      z <- round(sum(x<al)/length(x)*100,1)
      red <- paste("<font color=\"red\">")
      blue <- paste("<font color=\"blue\">")
      line <- paste("<h4>Percentage of rejected null hypotheses (in red):",z,"%</h4>")
      if(input$mu==10) line[2] <- HTML(paste("<h4>",blue,"&mu; = 10.0, so Null Hypothesis is true!</font></h4>"))
      else line[2] <- HTML(paste("<h4>",red,"&mu; = ",input$mu," &ne; 10.0, so Null Hypothesis is false!</font></h4>"))
      line
   })        
   
    
   output$plot1 <- renderPlot({
        x <- data()
        al <- as.numeric(input$alpha)
        brks <- hist(x,200,plot=F)$breaks
        hist(x,breaks=brks,xlab="p-value",ylab="",main="")
        hist(x[x<al],breaks=brks,add=T,col="red")    
   })
           

  output$explainText <- renderText({ 
        "<h5>We generate n observations from a normal distribution with mean &mu; and standard deviation 1.</h5>
         <h5>We then carry out a test of H<sub>0</sub>: &mu;=10 vs H<sub>a</sub>: &mu;&ne;10 and record the p-value.</h5>
         <h5>This is repeated 1000 times. We draw a histogram of the p-values and find the percentage of simulation runs with p&lt;&alpha;.</h5>
         <h5>If the test works as it should and if &mu;=10.0 the percentage should 100&alpha;%.</h5>
         <h5> As &mu; goes farther away from 10.0 the percentage should gow to 100%. </h5>"
  })

})

