library(shiny)

myblue <- colors()[589]

shinyServer(function(input, output) {

    x <- runif(50,0,1)
    y <- runif(50,0.6,0.9)
    x0 <- x
    colorId <- sample(0:1,size=50,replace=T)
    sizes <- runif(50,0.5,5)
    
    samples <- reactive({
       txt <- ""
       j <- 0
       if(input$method=="Simple Random Sample") smpls <- sample(1:50,size=10)
       if(input$method=="Stratified Sampling") smpls <- c(sample(c(1:50)[colorId==0],size=5),sample(c(1:50)[colorId==1],size=5))
       if(input$method=="Systematic Sampling") {
            k1 <- as.numeric(substring(input$whichrange,1,2))
            k2 <- as.numeric(substring(input$whichrange,4,5))
            smpls <- k1:k2
       }
       if(input$method=="Cluster Sampling") {       
          j <- sample(1:3,1)
          if(j==1) index <- c(1:50)[x<1/3]  
          if(j==2) index <- c(1:50)[x>1/3 & x<2/3]
          if(j==3) index <- c(1:50)[x>2/3]
          smpls <- 1:50
          if(length(index)<10) txt <- c("Not enough data in Cluster!","Start Over") 
          else smpls <- sample(index,size=10)
       }   
       list(smpls,txt,j)
    })
    
    data1 <- reactive({
        x[colorId==0] <- x[colorId==0] - x0[colorId==0]/1.9*input$step1/10
        x[colorId==1] <- x[colorId==1] + (1-x0[colorId==1])/1.9*input$step1/10        
        x
   })     

    data2 <- reactive({
        y0 <- y[samples()[[1]]]
        y1 <- (y0-0.6)/0.8
        y2 <- y0*(10-input$step2)/10+y1*input$step2/10
        y3 <- y
        y3[samples()[[1]]] <- y2
        y3
   })          
    
   output$text <- renderText({
        txt <- samples()[[2]]
        if(length(txt)==1) return("")
        paste("<h4>",txt[1],"</h4><h4>",txt[2],"</h4>")
   
   }) 
    
   output$text1 <- renderText({
        if(input$method=="Simple Random Sample") 
          txt <- paste("<h4>Simple Random Sample: picks any of the balls in the population</h4>")
        if(input$method=="Stratified Sampling") 
          txt <- paste("<h4>Stratified Sampling: divides balls by color, then selects from each color</h4>")
        if(input$method=="Systematic Sampling") 
          txt <- paste("<h4>Systematic Sampling: labels each ball from 1 to 50, chooses balls from selected range</h4>")
        if(input$method=="Cluster Sampling") 
          txt <- paste("<h4>Cluster Sampling: devides balls by location, randomly chooses location 
                and randomly chooses balls from that location</h4>")
        txt
   }) 
    
   output$plot <- renderPlot({
        plot(c(0, 1), c(0, 1), axes = F, xlab = "", ylab = "", type = "n")
        mtext("Population",3,cex=2)
        points(data1(),y,col=ifelse(colorId==0,"red","blue"),cex=sizes,pch=20)
        rect(0,0.52,1,1,lwd=3)
        if(input$method=="Systematic Sampling") 
            text(data1(),y+0.05,cex=1,as.character(1:50))
        if(input$method=="Cluster Sampling") {
            segments(1/3,0.52,1/3,1,lty=2)
            segments(2/3,0.52,2/3,1,lty=2)
            if(input$step2>0) {
              j <- samples()[[3]]
              if(j==1) rect(0,0.52,1/3,1,density=2)
              if(j==2) rect(1/3,0.52,2/3,1,density=2)
              if(j==3) rect(2/3,0.52,1,1,density=2)
            }  
        }    
        if(input$step1==10) segments(0.5,0.52,0.5,1,lty=2)

        if(input$step2>0) {
            points(data1(),data2(),col=ifelse(colorId==0,"red","blue"),cex=sizes,pch=20)
            if(input$method=="Systematic Sampling") 
                  text(data1(),data2()+0.05,cex=1,as.character(1:50))
            rect(0,0,1,0.4,lwd=3)
            text(0.5,0.45,"Sample",cex=2)               
        }
   })
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates various sampling methods
        <p>Click on Start Over! before using a different sampling method</h5>"
        
  })

})

