library(shiny)

shinyServer(function(input, output) {
       
  data <- reactive({
  
      mw <- input$mw*1000
      prepa <- (1+input$inflation/100/12)^(0:(25*12-1))*mw/12*0.29
         
      mix <- input$mixrange    
      mix <- c(mix[1],mix[2]-mix[1],100-mix[2])
      mix <- mix/sum(mix)
      low <- c(0.15,0.17,0.19)+0.03-0.015*input$esc
      high <- c(0.17,0.19,0.21)+0.03-0.015*input$esc
          
      ppalow <- sum(mix*low)*mw*(1+input$esc/100)^c(0:24) 
      ppalow <- rep(ppalow/12,each=12)
      ppahigh <- sum(mix*high)*mw*(1+input$esc/100)^c(0:24) 
      ppahigh <- rep(ppahigh/12,each=12)
      
      ppa <- (ppalow+ppahigh)/2
      
      mw <- mw*2/3
      
      cost <- (mix[1]*3+mix[2]*3.25+mix[3]*3.50)*mw
      loan <- (1+input$interest/100/12)^(12*input$years)*
                        (input$interest/100/12)*cost/((1+input$interest/100/12)^(12*input$years)-1)
            
      A <- matrix(0,4,3)
      colnames(A) <- c("Prepa","Buy","PPA")
      rownames(A) <- c("First Month Payment","Last Month Payment","Total Cost","Total Savings")
      A[1,] <- c(prepa[1],loan,ppa[1])
      A[2,] <- c(prepa[12*25],loan,ppa[12*25])
      A[3,] <- c(sum(prepa),12*input$years*loan,sum(ppa))
      A[4,] <- A[3,1]-A[3,]
      list(A,list(prepa,loan,ppalow,ppahigh,ppa))
  })
    html_data <- reactive({
        digits <- -2
        x <- data()[[1]]
        n <- dim(x)[1]
        m <- dim(x)[2]
        for (i in 1:m) {
            if (is.numeric(x[, i])) x[, i] <- round(x[, i], digits)
        }    
        DoRowNames <- ifelse(rownames(x)[1] == "1", FALSE, TRUE)
        top <- "<TABLE BORDER ALIGN=center>"
        if (DoRowNames) 
            nameline <- c("<TR> <TD> </TD>")
        else nameline <- c("<TR>")
        for (i in 1:m) nameline <- paste(nameline, paste("<TH>&nbsp;", 
            colnames(x)[i], "&nbsp;</TH>"), collapse = "")
        nameline <- paste(nameline, "</TR>")
        middle <- rep("<TR ALIGN=right>", n)
        for (j in 1:n) {
            if (DoRowNames) 
                middle[j] <- paste(middle[j], paste("<TH>&nbsp;", rownames(x)[j], 
                  "&nbsp;</TH>"), collapse = "")
            for (i in 1:m) middle[j] <- paste(middle[j], paste("<TD>&nbsp;", 
                x[j, i], "&nbsp;</TD>"), collapse = "")
            middle[j] <- paste(middle[j], "</TR>")
        }
        bottom <- "</Table>"
        c(top,nameline,middle,bottom)
   })    
   
   output$text1 <- renderText({
          if(input$mw<1500) mw <- paste(sprintf("%0.0f",input$mw*2/3),"Kw AC")
          else mw <- paste(sprintf("%0.0f",input$mw*2/3000),"Mw AC")
          line <-paste("<h4>System Size :",mw,"</h4>")
          mix <- input$mixrange    
          mix <- c(mix[1],mix[2]-mix[1],100-mix[2])
          mix <- round(100*mix/sum(mix))
          line[2] <- paste("<h4>Groundmount: ",mix[1],"%,  Roofmount: ",mix[2],"%, Carport: ",mix[3],"%</h4>",sep="")
          line
   })

   output$table2 <- renderText({
        html_data()
   })      
        
 
    output$myPlot <- renderPlot({
      y <- data()[[2]]
    
      par(mfrow=c(1,2))
      yr <- range(unlist(y))
      yr <- c(0.9*yr[1],1.2*yr[2])
      x <- seq(2015,2040,length=25*12)
      plot(x,y[[1]],ylim=yr,type="l",lwd=3,ylab="",main="Monthly Payment\n(likely range)",xlab="Year",cex=1.5)
      polygon(c(x,rev(x)),c(y[[1]]*0.9,rev(y[[1]]*1.2)),col="orange")
      lines(x,y[[1]],lwd=3,col="red")
      polygon(c(x,rev(x)),c(y[[4]],rev(y[[3]])),col="lightblue")
      lines(x,y[[5]],lwd=3,col="blue")      
      rect(2015,y[[2]]-yr/100,2015+input$years,y[[2]]+yr/100,col="green",lwd=3)
      legend(2015,yr[2],c("Prepa","Buy","PPA"),lty=c(1,1,1),lwd=3,col=c("red","green","blue"))  
      
      y1 <- cumsum(y[[1]]-y[[5]])/1e6
      k <- 12*input$years
      loan <- c(rep(y[[2]],k),rep(0,length(y1)-k))
      y2 <- cumsum(y[[1]]-loan)/1e6
      y3 <- cumsum(y[[5]]-loan)/1e6
      yr <- range(c(y1,y2,y2))
      plot(x,y1,type="l",lwd=3,col="red",ylim=yr,xlab="Year",ylab="",
            main="Accumulated Savings\n(in million $)")
      lines(x,y2,lwd=3,col="blue")
      lines(x,y3,lwd=3,col="green")
      legend(2015,max(yr),c("PPA over Prepa","Buy over Prepa","Buy over PPA"),lty=rep(1,3),lwd=3,col=c("red","blue","green"))
  })

  output$explainText <- renderText({ 
        "<h5>How to use the Calculator:</h5>
         <h5>Move the sliders to reflect your project (amount of electricity desired etc.)</h5>
         <h5>The slider for the system type is for the mix of installations. The default is 33% groundmount, 33%
         roofmount and 33% carport. For a 100% groundmount system move both sliders all the way to the right, for a 100% carport 
         move them all the way to the left</h5>     
         <h5>Now the table and the graphs show the relevant numbers</h5>
         <h5>Change the settings to see the effect on the payments</h5>
         <h5>The Prepa case assumes a price of 29c per KW, the graph on the left shows a range of 26c to 35c</h5>"
         
  })
})

