library(shiny)

shinyServer(function(input, output,session) {
          
   red <- paste("<font color=\"red\">")             
   blue <- paste("<font color=\"blue\">") 
   green <- paste("<font color=\"lightgreen\">")             
   gray <- paste("<font color=\"gray\">") 
   npoints <- 250
   nmult <- 5
   
   h <- function(a) {      
      a <- round(a,3)
      n <- length(a)
      
      if(sum(abs(a))==0) return("0")
      if(a[1]==0) {
          if(abs(a[2])==1) {
              if(a[2]==-1) txt <- "- x" 
              else txt <- " x" 
          }
          else txt <- paste(a[2],"x") 
      } 
      else {
          if(a[2]==0) txt <- paste(a[1])
          else {
             sgn <- ifelse(a[2]>=0," +"," -") 
             txt <- paste(a[1],sgn,abs(a[2]),"x") 
          }   
      }    
      if(n>2) {
          for(i in 3:n) {
              if(a[i]==0) next
              sgn <- ifelse(a[i]>0," +"," -")
              txt2 <- ifelse(abs(a[i])==1,"",abs(a[i]))
              txt <- paste(txt," ",sgn," ",txt2," x<sup>",i-1,"</sup>",sep="")
          }    
      }
      txt    
   }    
   
   poly <- function(a,p) {

         A <- matrix(0,4,5)
         A[1,1:2] <- c(a[1]-a[2]*p,a[2])
         A[2,1:3] <- A[1,1:3]+a[3]*c(p^2,-2*p,1)
         A[3,1:4] <- A[2,1:4]+a[4]*c(p^3,-3*p^2,3*p^2,1)   
         A[4,] <- A[3,]+a[5]*c(p^4,-4*p^3,6*p^2,-4*p,1)
         A
   }
       
   data <- reactive({
        if(input$f=="log(x+1)") {        
            x <- seq(-0.9,2,length=npoints)
            p <- x[nmult*input$k]
            y  <- log(x+1)
            p0 <- log(p+1)
            p1 <- 1/(1+p)
            p2 <- -1/(1+p)^2
            p3 <- 2/(1+p)^3
            p4 <- -6/(1+p)^4

        }
        if(input$f=="sin(x)") {
            x <- seq(0,6,length=npoints)
            p <- x[nmult*input$k]
            y  <- sin(x)
            p0 <- sin(p)
            p1 <- cos(p)
            p2 <- -sin(p)
            p3 <- -cos(p)
            p4 <- sin(p)

        }        
        if(input$f=="exp(-x^2)") {
            x <- seq(-2.5,2.5,length=npoints)
            p <- x[nmult*input$k]
            y  <- exp(-x^2)
            p0 <- exp(-p^2)
            p1 <- -2*p*exp(-p^2)
            p2 <- 2*(-1+2*p^2)*exp(-p^2)
            p3 <- 4*(-p^2+2*p-2*p^3)*exp(-p^2)
            p4 <- 8*(-p+1-5*p^2+p^3+2*p^4)*exp(-p^2)

        } 
        if(input$f=="Enter you own!") {
            x <- seq(input$from,input$to,length=npoints)
            i <- nmult*input$k
            p <- x[i]
            h <- x[2]-x[1]
            f<-function(x) {
                eval(parse(text=input$fun))
            }
            y <- f(x)
            p0 <- y[i]
            p1 <- (y[i+1]-y[i])/h
            p2 <- (y[i-1]-2*y[i]+y[i+1])/h^2
            p3 <- (y[i+2]-3*y[i+1]+3*y[i]-y[i-1])/h^3
            p4 <- (y[i+2]-4*y[i+1]+6*y[i]-4*y[i-1]+y[i-2])/h^4
        
        }       

              
        yr <- c(min(y)-(max(y)-min(y)/3),max(y)+(max(y)-min(y)/3))
        
        list(cbind(x,y),c(p0,p1,p2,p3,p4),p,yr)
   })     
   

   output$text <- renderText({
      coef <- poly(data()[[2]]/c(1,1,2,6,24),data()[[3]])
     
      if(input$f=="Enter you own!") line <- paste("<h4>f(x) = ",input$fun,"</h4>")
      else line <- paste("<h4>f(x)=",input$f,"</h4>")
      line[2] <- paste("<h4>Approximation Point: x =",round(data()[[3]],2),"</h4>")

      line[3] <- paste("<h4>",blue,"Linear: p<sub>1</sub>(x) =",
                      h(coef[1,1:2]),"</h4>")      
      
      line[4] <- paste("<h4>",green,"Quadratic: p<sub>2</sub>(x) =",
                h(coef[2,1:3]),"</h4>")      
                
      line[5] <- paste("<h4>",red,"Cubic: p<sub>3</sub>(x) =",
                h(coef[3,1:4]),"</h4>")      

      line[6] <- paste("<h4>",gray,"Biquadratic: p<sub>4</sub>(x) =",
                 h(coef[4,]),"</h4>")      

      if(input$deg=="All") return(line)
      if(input$deg=="Linear") return(line[c(1,2,3)])
      if(input$deg=="Quadratic") return(line[c(1,2,4)])
      if(input$deg=="Cubic") return(line[c(1,2,5)])
      if(input$deg=="Biquadratic") return(line[c(1,2,6)])
   })  
     
    
   output$plot <- renderPlot({
        x <- data()[[1]][,1]
        y <- data()[[1]][,2]
        yr <- data()[[4]]
        plot(x,y,ylim=yr,xlab="x",ylab="",type="l",lwd=3)
        points(x[nmult*input$k],y[nmult*input$k],pch=20,cex=2)
        a <- data()[[2]]
        p <- data()[[3]]
        y <- a[1]+a[2]*(x-p)
        if(input$deg=="All" | input$deg=="Linear") lines(x,y,lwd=1,col="blue")
        y <- y+a[3]/2*(x-p)^2
        if(input$deg=="All" | input$deg=="Quadratic") lines(x,y,lwd=1,col="green")
        y <- y+a[4]/6*(x-p)^3
        if(input$deg=="All" | input$deg=="Cubic") lines(x,y,lwd=1,col="red")
        y <- y+a[5]/24*(x-p)^4
        if(input$deg=="All" | input$deg=="Biquadratic") lines(x,y,lwd=1,col="gray")        
   })
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the Taylor expansion </h5>"
        
  })

})

