library(shiny)

shinyServer(function(input, output,session) {
           
   trans <- function(x,l) {
      l <- (c(10:0)/10)[l+1]
      if(l==0) return(log(x))
      (x^l-1)/l             
   } 
   output$plot1 <- renderPlot({
        if(input$dataset=="Cancer Survival") {
            par(mfrow=c(1,2))
            x <- read.table("cancer.R",header=TRUE)
            for(l in input$step) {
                y <- trans(x[,1],l)
                fit <- aov(y~x[,2])
                boxplot(y~x[,2])
                qqnorm(residuals(fit))
                qqline(residuals(fit))
            }
        }        
        if(input$dataset=="Wells") {
            par(mfrow=c(1,2))
            x <- read.table("wells.R",header=TRUE)
            for(l in input$step) {
                y <- trans(x[,2],l)
                fit <- aov(y~x[,1])
                boxplot(y~x[,1])
                qqnorm(residuals(fit))
                qqline(residuals(fit))
            }                
        }      
        if(input$dataset=="Mammals") {
            par(mfrow=c(2,2))
            mammals <- read.table("mammals.R",header=TRUE)
            for(l in input$step) {
                x <- trans(mammals[,1],l)
                y <- trans(mammals[,2],l)
                boxplot(x,main="Body",horizontal=T)
                eplot()
                plot(x,y,xlab="Body",ylab="Brain")
                boxplot(y,main="Brain")  
            }    
        }              
   })
           

  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the idea of transformations 
        <br>it runs a movie, starting with the data itself and then a series of transformations</h5>"
  })

})

