library(shiny)

shinyServer(function(input, output,session) {
        
 
   pvalues <- reactive({
       x1 <- matrix(rnorm(input$n1*1e3,10,3),nrow=1e3)
       xbar1 <- apply(x1,1,mean)    
       v1 <- apply(x1,1,var)    
       x2 <- matrix(rnorm(input$n2*1e3,input$mu2,input$s2),nrow=1e3)
       xbar2 <- apply(x2,1,mean)    
       v2 <- apply(x2,1,var)    
       sp <- sqrt((input$n1-1)*v1+(input$n2-1)*v2)/sqrt(input$n1+input$n2-2)
       T <-  (xbar1-xbar2)/sp/sqrt(1/input$n1+1/input$n2)
       2*(1-pt(abs(T),input$n1+input$n2-2))
   })     
   
 
   output$text1 <- renderText({
          x <- pvalues()
          a <- sum(x<as.numeric(input$alpha))/length(x)*100
          paste("<h4>Rejected Null Hypotheses:",sprintf("%0.1f",a),"%</h4>")
   })        
   
    
   output$plot1 <- renderPlot({
        x <- pvalues()        
        al <- as.numeric(input$alpha)
        brks <- hist(x,200,plot=F)$breaks
        hist(x,breaks=brks,xlab="p-value",ylab="",main="")
        hist(x[x<al],breaks=brks,add=T,col="red") 
   })
   output$plot2 <- renderPlot({
        for(i in input$step) {
          x <- rnorm(input$n1,10,3)
          y <- rnorm(input$n2,input$mu2,input$s2)
          boxplot(list('Sample 1'=x,'Sample 2'=y),main="Example Case")
        }  
   })           

  output$explainText <- renderText({ 
        mu <- paste(HTML("&mu;"))
        sigma <- paste(HTML("&sigma;"))
        line <- ""
        line[1] <- "<h5>Explanation: This demo illustrates the twosample t test</h5>"
        line[2] <- "<h5>We generate two datasets. One from a normal distribution with mean 10 and standard deviation 3 </h5>"
        line[3] <- paste("<h5>and the other from a normal distribution with mean ",mu," and standard deviation ",sigma,"</h5>",sep="")
        line[4] <- paste("<h5>Then we carry out the test H<sub>0</sub>: ",mu,"<sub>1</sub> = ",mu,"<sub>2</sub> vs H<sub>a</sub>: ",mu,"<sub>1</sub> &ne; ",mu,"<sub>2</sub></h5>",sep="")
        line  
  })

})

